//Importing the required functions from the d3 package.
import {
  select,
  csv,
  pie,
  arc,
  scaleOrdinal,
  schemeCategory10,
} from 'd3';
//we are creating a new svg element with specified length and width as you can see below.
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Using the below code, we are adding the "Rs" tag to the numbered values present in IncomeOfFinancialYear column.
const numberFormat = (number) =>
  Math.abs(Number(number)) + ' Rs';

//Here we are sorting the values of IncomeOfFinancialYear on the pie chart.
const render = (data) => {
  const pieData = pie()
    .sort(null)
    .value((d) => d.GDFOfFinancialYear)(data);
  //Here we are using scaleOrdinal to convert the values into ordinals.
  const colors = scaleOrdinal(schemeCategory10);
  //Using the below code, we are creating an arc with the inner radius zero and outer radius 200
  const segments = arc()
    .innerRadius(0)
    .outerRadius(200)
    .padAngle(0)
    .padRadius(0);

  const sections = svg
    .append('g')
    //Here we are giving the location where the pie data should be represented in the X and Y axis.
    .attr('transform', `translate(230,230)`)
    .selectAll('path')
    .data(pieData);
  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) => colors(d.data.GDFOfFinancialYear))
    .on('mouseover', (d) =>
      console.log(d.data.GDFOfFinancialYear),
    );
  //Now, we are creating the piechart and giving the location on the XY plane to append/add the svg
  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,50)')
    .selectAll('.legends')
    .data(pieData);
  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`;
    });
  //We are now giving the rectangle shape and its size to represent different states.
  //And appending it in the visualization
  legend
    .append('rect')
    .attr('width', 22)
    .attr('height', 25)
    //Here we are filling the rectangles based on the Income of the states
    .attr('fill', (d) => colors(d.data.GDFOfFinancialYear));
  //In the above part, the rectangles were created and they are illed with colors
  //Now the next thing is to append each color with a state from the sorted data
  //The below code does that. It will append the names of the top 10 states from the sorted data.
  legend
    .append('text')
    .attr('x', 28)
    .attr('y', 20)
    .attr('class', 'legend_text')
    .text((d) => d.data.StatesOfINDIA);
  //The result of above code will be containing rectangles filled with colors and the names of the states beside them.
  //Now, using the below code, we will append value of the income of each state respectively.
  legend
    .append('text')
    .attr('x', 260)
    .attr('y', 20)
    .attr('class', 'legend_value')
    .text((d) => numberFormat(d.data.GDFOfFinancialYear));
};
// Here we are assigning the url of our gist to the variable named csvURL
const csvUrl =
  'https://raw.githubusercontent.com/siddhartha-gurram/piechart/main/GDP-All-States.csv';
//As mentioned above, the URL has been assigned to the variable "csvUrl".
//Now using the below code, we are loading the data present in the link.
csv(csvUrl).then((csvData) => {
  /* If we observe, there are nearly 32 rows in the data but here we are getting the top 10 states only. 
  This can be acheived by sorting the given data. 
  So, the sorting of the data is done in this step.
  And once the sorting is done, we are slicing the sorted data from 1 to 10 using the render function and then represent in our visualization.
  */
  csvData.sort(
    (data1, data2) =>
      data2.GDFOfFinancialYear - data1.GDFOfFinancialYear,
  );
  if (csvData) {
    render(csvData.slice(0, 15));
  }
});
